package sun.java2d.cmm;

import java.awt.color.ICC_Profile;
import java.awt.color.CMMException;
import sun.security.action.GetPropertyAction;
import java.security.AccessController;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.security.PrivilegedAction;
import java.awt.color.ColorSpace;

public class CMSManager
{
    public static ColorSpace GRAYspace;
    public static ColorSpace LINEAR_RGBspace;
    private static PCMM cmmImpl;
    
    public CMSManager() {
    }
    
    public static synchronized PCMM getModule() {
        if (CMSManager.cmmImpl != null) {
            return CMSManager.cmmImpl;
        }
        CMSManager.cmmImpl = ((CMMServiceProvider)AccessController.doPrivileged(new PrivilegedAction<CMMServiceProvider>() {
            @Override
            public CMMServiceProvider run() {
                String property = System.getProperty("sun.java2d.cmm", "sun.java2d.cmm.lcms.LcmsServiceProvider");
                ServiceLoader<CMMServiceProvider> loadInstalled = ServiceLoader.loadInstalled(CMMServiceProvider.class);
                CMMServiceProvider cmmServiceProvider = null;
                Iterator<Object> iterator = loadInstalled.iterator();
                while (iterator.hasNext() && !(cmmServiceProvider = (CMMServiceProvider)iterator.next()).getClass().getName().equals(property)) {}
                return cmmServiceProvider;
            }
        })).getColorManagementModule();
        if (CMSManager.cmmImpl == null) {
            throw new CMMException("Cannot initialize Color Management System.No CM module found");
        }
        if ((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.cmm.trace")) != null) {
            CMSManager.cmmImpl = new CMSManager.CMMTracer(CMSManager.cmmImpl);
        }
        return CMSManager.cmmImpl;
    }
    
    static synchronized boolean canCreateModule() {
        return CMSManager.cmmImpl == null;
    }
    
    static {
        CMSManager.cmmImpl = null;
    }
    
    public static class CMMTracer implements PCMM
    {
        PCMM tcmm;
        String cName;
        
        public CMMTracer(PCMM tcmm) {
            this.tcmm = tcmm;
            this.cName = tcmm.getClass().getName();
        }
        
        @Override
        public Profile loadProfile(byte[] array) {
            System.err.print(this.cName + ".loadProfile");
            Profile loadProfile = this.tcmm.loadProfile(array);
            System.err.printf("(ID=%s)\n", new Object[] { loadProfile.toString() });
            return loadProfile;
        }
        
        @Override
        public void freeProfile(Profile profile) {
            System.err.printf(this.cName + ".freeProfile(ID=%s)\n", new Object[] { profile.toString() });
            this.tcmm.freeProfile(profile);
        }
        
        @Override
        public int getProfileSize(Profile obj) {
            System.err.print(this.cName + ".getProfileSize(ID=" + obj + ")");
            int profileSize = this.tcmm.getProfileSize(obj);
            System.err.println("=" + profileSize);
            return profileSize;
        }
        
        @Override
        public void getProfileData(Profile obj, byte[] array) {
            System.err.print(this.cName + ".getProfileData(ID=" + obj + ") ");
            System.err.println("requested " + array.length + " byte(s)");
            this.tcmm.getProfileData(obj, array);
        }
        
        @Override
        public int getTagSize(Profile profile, int n) {
            System.err.printf(this.cName + ".getTagSize(ID=%x, TagSig=%s)", new Object[] { profile, signatureToString(n) });
            int tagSize = this.tcmm.getTagSize(profile, n);
            System.err.println("=" + tagSize);
            return tagSize;
        }
        
        @Override
        public void getTagData(Profile profile, int n, byte[] array) {
            System.err.printf(this.cName + ".getTagData(ID=%x, TagSig=%s)", new Object[] { profile, signatureToString(n) });
            System.err.println(" requested " + array.length + " byte(s)");
            this.tcmm.getTagData(profile, n, array);
        }
        
        @Override
        public void setTagData(Profile obj, int i, byte[] array) {
            System.err.print(this.cName + ".setTagData(ID=" + obj + ", TagSig=" + i + ")");
            System.err.println(" sending " + array.length + " byte(s)");
            this.tcmm.setTagData(obj, i, array);
        }
        
        @Override
        public ColorTransform createTransform(ICC_Profile icc_Profile, int n, int n2) {
            System.err.println(this.cName + ".createTransform(ICC_Profile,int,int)");
            return this.tcmm.createTransform(icc_Profile, n, n2);
        }
        
        @Override
        public ColorTransform createTransform(ColorTransform[] array) {
            System.err.println(this.cName + ".createTransform(ColorTransform[])");
            return this.tcmm.createTransform(array);
        }
        
        private static String signatureToString(int n) {
            return String.format("%c%c%c%c", new Object[] { Character.valueOf((char)(0xFF & n >> 24)), Character.valueOf((char)(0xFF & n >> 16)), Character.valueOf((char)(0xFF & n >> 8)), Character.valueOf((char)(0xFF & n)) });
        }
    }
}
